﻿using System.Globalization;
using System.Reflection;
using FlexCel.Core;
using FlexCel.Pdf;
using FlexCel.Render;
using FlexCel.Report;
using FlexCel.XlsAdapter;

namespace FlexcelTestApp;

[Register ("AppDelegate")]
public class AppDelegate : UIApplicationDelegate {
	public override UIWindow? Window {
		get;
		set;
	}

	public override bool FinishedLaunching (UIApplication application, NSDictionary launchOptions)
	{
		// create a new window instance based on the screen size
		Window = new UIWindow (UIScreen.MainScreen.Bounds);

		// create a UIViewController with a single UILabel
		var vc = new UIViewController ();
		UIButton button = new UIButton(Window!.Frame);
		button.SetTitle("Cold start", UIControlState.Normal) ;
        button.TouchDown += Button_TouchDown;
		vc.View!.AddSubview (button);
		Window.RootViewController = vc;

		// make the window visible
		Window.MakeKeyAndVisible ();

		return true;
	}

    private void Button_TouchDown(object? sender, EventArgs e)
    {
        ColdStart();
    }

    private void ColdStart()
    {
        Assembly assembly = Assembly.GetExecutingAssembly();
        bool startedSuccessfully = true;
        using (Stream coldStartTemplateStream = assembly.GetManifestResourceStream("FlexcelTestApp.test.xlsx"))
        {
            using (MemoryStream flexCelReportResultStream = new MemoryStream())
            {
                XlsFile xls = null;
                try
                {
                    using (FlexCelReport fr = new FlexCelReport(true))
                    {
                        fr.DisableSQLValidation = true;
                        fr.Run(coldStartTemplateStream, flexCelReportResultStream);
                        flexCelReportResultStream.Position = 0;

                        xls = new XlsFile(true);
                        string pathToSave = Path.Combine(Directory.GetCurrentDirectory(), "result.xlsx");
                        xls.Open(flexCelReportResultStream);
                        xls.Save(pathToSave);
                    }
                }
                catch (Exception e)
                {
                    string error = e.StackTrace;
                    startedSuccessfully = false;
                }

                if (xls != null && startedSuccessfully)
                {
                    startedSuccessfully = MakeInMemoryPdf(xls);
                }
            }
        }

        if (startedSuccessfully)
        {
        }
    }

    private bool MakeInMemoryPdf(XlsFile xls)
    {
        FlexCelConfig.GraphicFramework = FlexCelGraphicFramework.Native;
        FlexCelTrace.OnError += HandleOnFlexcelError;
        bool result = true;
        using (MemoryStream pdfStream = new MemoryStream())
        {
            using (FlexCelPdfExport pdf = new FlexCelPdfExport(xls, true))
            {
                try
                {

                    string pathToSave = Path.Combine(Directory.GetCurrentDirectory(), "result.pdf");
                    pdf.FontMapping = TFontMapping.ReplaceAllFonts;
                    pdf.GetFontData += HandleOnGetFontData;
                    pdf.BeforeGeneratePage += HandleOnBeforeGeneratingPage;
                    pdf.OnFontFolderNotFound = TPdfFontFolderNotFoundAction.Ignore;
                    pdf.UnlicensedFontAction = TUnlicensedFontAction.Ignore;
                    pdf.AfterGeneratePage += HandleOnAfterGeneratingPage;
                    pdf.GetFontFolder += HandleOnGetFontFolder;

                    pdf.Export(pdfStream);
                    pdf.Export(pathToSave);
                    result = true;
                }

                catch (Exception e)
                {
                    result = false;
                }
            }
        }

        return result;
    }

    private void HandleOnAfterGeneratingPage(object sender, PageEventArgs e)
    {

    }

    private void HandleOnGetFontFolder(object sender, GetFontFolderEventArgs e)
    {
    }

    private void HandleOnBeforeGeneratingPage(object sender, PageEventArgs e)
    {
    }

    private void HandleOnGetFontData(object sender, GetFontDataEventArgs e)
    {
    }

    private void HandleOnFlexcelError(TFlexCelErrorInfo e)
    {
    }
}

